@@ -20,12 +20,14 @@ import (
 	"bytes"
 	"encoding/binary"
 	"io"
+	"strconv"
 
 	"github.com/apache/arrow-go/v18/arrow"
 	"github.com/apache/arrow-go/v18/arrow/array"
 	"github.com/apache/arrow-go/v18/arrow/bitutil"
 	"github.com/apache/arrow-go/v18/arrow/memory"
 	"github.com/apache/arrow-go/v18/parquet"
+	"github.com/apache/arrow-go/v18/parquet/internal/debug"
 	"github.com/apache/arrow-go/v18/parquet/internal/encoding"
 	"github.com/apache/arrow-go/v18/parquet/metadata"
 	"github.com/apache/arrow-go/v18/parquet/schema"
@@ -76,6 +78,8 @@ type ColumnChunkWriter interface {
 	LevelInfo() LevelInfo
 	SetBitsBuffer(*memory.Buffer)
 	HasBitsBuffer() bool
+
+	GetBloomFilter() metadata.BloomFilterBuilder
 }
 
 func computeLevelInfo(descr *schema.Column) (info LevelInfo) {
@@ -111,6 +115,7 @@ type columnWriter struct {
 
 	pageStatistics  metadata.TypedStatistics
 	chunkStatistics metadata.TypedStatistics
+	bloomFilter     metadata.BloomFilterBuilder
 
 	// total number of values stored in the current data page. this is the maximum
 	// of the number of encoded def levels or encoded values. for
@@ -172,6 +177,7 @@ func newColumnWriterBase(metaData *metadata.ColumnChunkMetaDataBuilder, pager Pa
 
 	ret.reset()
 
+	ret.initBloomFilter()
 	return ret
 }
 
@@ -331,16 +337,30 @@ func (w *columnWriter) buildDataPageV1(defLevelsRLESize, repLevelsRLESize, uncom
 		data = w.uncompressedData.Bytes()
 	}
 
+	firstRowIndex := int64(w.rowsWritten)
+
 	// write the page to sink eagerly if there's no dictionary or if dictionary encoding has fallen back
 	if w.hasDict && !w.fallbackToNonDict {
 		pageSlice := make([]byte, len(data))
 		copy(pageSlice, data)
-		page := NewDataPageV1WithStats(memory.NewBufferBytes(pageSlice), int32(w.numBufferedValues), w.encoding, parquet.Encodings.RLE, parquet.Encodings.RLE, uncompressed, pageStats)
+		page := NewDataPageV1WithConfig(memory.NewBufferBytes(pageSlice), parquet.Encodings.RLE, parquet.Encodings.RLE, DataPageConfig{
+			Num:              int32(w.numBufferedValues),
+			Encoding:         w.encoding,
+			UncompressedSize: uncompressed,
+			Stats:            pageStats,
+			FirstRowIndex:    firstRowIndex,
+		})
 		w.totalCompressedBytes += int64(page.buf.Len()) // + size of Pageheader
 		w.pages = append(w.pages, page)
 	} else {
 		w.totalCompressedBytes += int64(len(data))
-		dp := NewDataPageV1WithStats(memory.NewBufferBytes(data), int32(w.numBufferedValues), w.encoding, parquet.Encodings.RLE, parquet.Encodings.RLE, uncompressed, pageStats)
+		dp := NewDataPageV1WithConfig(memory.NewBufferBytes(data), parquet.Encodings.RLE, parquet.Encodings.RLE, DataPageConfig{
+			Num:              int32(w.numBufferedValues),
+			Encoding:         w.encoding,
+			UncompressedSize: uncompressed,
+			Stats:            pageStats,
+			FirstRowIndex:    firstRowIndex,
+		})
 		defer dp.Release()
 		return w.WriteDataPage(dp)
 	}
@@ -374,9 +394,16 @@ func (w *columnWriter) buildDataPageV2(defLevelsRLESize, repLevelsRLESize, uncom
 	nullCount := int32(pageStats.NullCount)
 	defLevelsByteLen := int32(defLevelsRLESize)
 	repLevelsByteLen := int32(repLevelsRLESize)
+	firstRowIndex := int64(w.rowsWritten)
 
-	page := NewDataPageV2WithStats(memory.NewBufferBytes(combined.Bytes()), numValues, nullCount, numRows, w.encoding,
-		defLevelsByteLen, repLevelsByteLen, uncompressed, w.pager.HasCompressor(), pageStats)
+	page := NewDataPageV2WithConfig(memory.NewBufferBytes(combined.Bytes()), nullCount, numRows, defLevelsByteLen, repLevelsByteLen,
+		w.pager.HasCompressor(), DataPageConfig{
+			Num:              numValues,
+			Encoding:         w.encoding,
+			UncompressedSize: uncompressed,
+			Stats:            pageStats,
+			FirstRowIndex:    firstRowIndex,
+		})
 	if w.hasDict && !w.fallbackToNonDict {
 		w.totalCompressedBytes += int64(page.buf.Len()) // + sizeof pageheader
 		w.pages = append(w.pages, page)
@@ -395,9 +422,12 @@ func (w *columnWriter) FlushBufferedDataPages() (err error) {
 		}
 	}
 
-	for _, p := range w.pages {
+	for i, p := range w.pages {
 		defer p.Release()
 		if err = w.WriteDataPage(p); err != nil {
+			// To keep pages in consistent state,
+			// remove the pages that will be released using above defer call.
+			w.pages = w.pages[i+1:]
 			return err
 		}
 	}
@@ -407,10 +437,14 @@ func (w *columnWriter) FlushBufferedDataPages() (err error) {
 
 func (w *columnWriter) writeLevels(numValues int64, defLevels, repLevels []int16) int64 {
 	toWrite := int64(0)
+	maxDefLevel := w.descr.MaxDefinitionLevel()
+
 	// if the field is required and non-repeated, no definition levels
-	if defLevels != nil && w.descr.MaxDefinitionLevel() > 0 {
+	if defLevels != nil && maxDefLevel > 0 {
 		for _, v := range defLevels[:numValues] {
-			if v == w.descr.MaxDefinitionLevel() {
+			debug.Assert(v <= maxDefLevel, "columnwriter: invalid definition level "+
+				strconv.Itoa(int(v))+" for column "+w.descr.Path())
+			if v == maxDefLevel {
 				toWrite++
 			}
 		}
@@ -675,3 +709,29 @@ func (w *columnWriter) maybeReplaceValidity(values arrow.Array, newNullCount int
 	defer data.Release()
 	return array.MakeFromData(data)
 }
+
+func (w *columnWriter) initBloomFilter() {
+	path := w.descr.Path()
+	if !w.props.BloomFilterEnabledFor(path) {
+		return
+	}
+
+	maxFilterBytes := w.props.MaxBloomFilterBytes()
+	ndv := w.props.BloomFilterNDVFor(path)
+	fpp := w.props.BloomFilterFPPFor(path)
+	// if user specified the column NDV, we can construct the bloom filter for it
+	if ndv > 0 {
+		w.bloomFilter = metadata.NewBloomFilterFromNDVAndFPP(uint32(ndv), fpp, maxFilterBytes, w.mem)
+	} else if w.props.AdaptiveBloomFilterEnabledFor(path) {
+		numCandidates := w.props.BloomFilterCandidatesFor(path)
+		// construct adaptive bloom filter writer
+		w.bloomFilter = metadata.NewAdaptiveBlockSplitBloomFilter(uint32(maxFilterBytes), numCandidates, fpp, w.descr, w.mem)
+	} else {
+		// construct a bloom filter using the max size
+		w.bloomFilter = metadata.NewBloomFilter(uint32(maxFilterBytes), uint32(maxFilterBytes), w.mem)
+	}
+}
+
+func (w *columnWriter) GetBloomFilter() metadata.BloomFilterBuilder {
+	return w.bloomFilter
+}
